/*-----------------------------------------------------------------------*/
/* Low level disk I/O module skeleton for Petit FatFs (C)ChaN, 2009      */
/*-----------------------------------------------------------------------*/

#include "spi.h"
#include "mmc.h"
#include "diskio.h"


/*-----------------------------------------------------------------------*/
/* Initialize Disk Drive                                                 */
/*-----------------------------------------------------------------------*/

DSTATUS disk_initialize(void)
{
	return mmc_init();
}



/*-----------------------------------------------------------------------*/
/* Read Partial Sector                                                   */
/*-----------------------------------------------------------------------*/

DRESULT disk_readp(
	BYTE* buff,			/* Pointer to the destination object */
	DWORD sector,		/* Sector number (LBA) */
	WORD offset,			/* Offset in the sector */
	WORD count			/* Byte count (bit15:destination) */
	)
{
	DRESULT res;
	BYTE rc;
	WORD bc;


	if (!(CardType & CT_BLOCK)) sector *= 512;		/* Convert to byte address if needed */

	res = RES_ERROR;
	if (send_cmd(CMD17, sector) == 0) {		/* READ_SINGLE_BLOCK */

		bc = 40000;
		do {							/* Wait for data packet */
			rc = spi_recv();
		} while (rc == 0xFF && --bc);

		if (rc == 0xFE) {				/* A data packet arrived */
			bc = 514 - offset - count;

			/* Skip leading bytes */
			if (offset) {
				do spi_recv(); while (--offset);
			}

			/* Receive a part of the sector */
			if (buff) {	/* Store data to the memory */
				do {
					*buff++ = spi_recv();
				} while (--count);
			}

			// else {	/* Forward data to the outgoing stream (depends on the project) */
			//	do {
			//		FORWARD(spi_recv());
			//	} while (--count);
			//}

			/* Skip trailing bytes and CRC */
			do spi_recv(); while (--bc);

			res = RES_OK;
		}
	}

	DESELECT_MMC();
	spi_recv();

	return res;
}



/*-----------------------------------------------------------------------*/
/* Write Partial Sector                                                  */
/*-----------------------------------------------------------------------*/
#if _USE_WRITE
DRESULT disk_writep(
	BYTE* buff,		/* Pointer to the data to be written, NULL:Initiate/Finalize write operation */
	DWORD sc		/* Sector number (LBA) or Number of bytes to send */
	)
{
	DRESULT res;


	if (!buff) {
		if (sc) {
			res = 1;
			// Initiate write process

		}
		else {

			// Finalize write process

		}
	}
	else {

		// Send data to the disk

	}

	return res;
}
#endif
