/*-------------------------------------------------------------------------*/
/* PFF - Low level disk control module for AVR            (C)ChaN, 2010    */
/*-------------------------------------------------------------------------*/

#include "pff.h"
#include "spi.h"
#include "mmc.h"

BYTE mmc_init()
{
	BYTE cmd, ty, ocr[4];
	UINT tmr;

	spi_init_low();

	DESELECT_MMC();
	_delay_ms(10);

	/* 80 dummy clocks with CS=H */
	for (int n = 10; n; n--) spi_recv();

	ty = 0;
	if (send_cmd(CMD0, 0) == 1) {			/* Enter Idle state */
		if (send_cmd(CMD8, 0x1AA) == 1) {	/* SDv2 */
			for (int n = 0; n < 4; n++) ocr[n] = spi_recv();		/* Get trailing return value of R7 resp */
			if (ocr[2] == 0x01 && ocr[3] == 0xAA) {			/* The card can work at vdd range of 2.7-3.6V */
				for (tmr = 10000; tmr && send_cmd(ACMD41, 1UL << 30); tmr--) _delay_ms(1);	/* Wait for leaving idle state (ACMD41 with HCS bit) */
				if (tmr && send_cmd(CMD58, 0) == 0) {		/* Check CCS bit in the OCR */
					for (int n = 0; n < 4; n++) ocr[n] = spi_recv();
					ty = (ocr[0] & 0x40) ? CT_SD2 | CT_BLOCK : CT_SD2;	/* SDv2 (HC or SC) */
				}
			}
		}
		else {							/* SDv1 or MMCv3 */
			if (send_cmd(ACMD41, 0) <= 1) 	{
				ty = CT_SD1; cmd = ACMD41;	/* SDv1 */
			}
			else {
				ty = CT_MMC; cmd = CMD1;	/* MMCv3 */
			}
			for (tmr = 10000; tmr && send_cmd(cmd, 0); tmr--) _delay_ms(1);	/* Wait for leaving idle state */
			if (!tmr || send_cmd(CMD16, 512) != 0)			/* Set R/W block length to 512 */
				ty = 0;
		}
	}

	CardType = ty;
	DESELECT_MMC();
	spi_recv();

	return ty ? 0 : 1;
}

/*-----------------------------------------------------------------------*/
/* Send a command packet to MMC                                          */
/*-----------------------------------------------------------------------*/

BYTE send_cmd(
	BYTE cmd,		/* 1st byte (Start + Index) */
	DWORD arg		/* Argument (32 bits) */
	)
{
	BYTE n, res;

	if (cmd & 0x80) {	/* ACMD<n> is the command sequense of CMD55-CMD<n> */
		cmd &= 0x7F;
		res = send_cmd(CMD55, 0);
		if (res > 1) return res;
	}

	SELECT_MMC();

	/* Send a command packet */
	spi_send(cmd);                /* Start + Command index */
	spi_send((BYTE)(arg >> 24));  /* Argument[31..24] */
	spi_send((BYTE)(arg >> 16));  /* Argument[23..16] */
	spi_send((BYTE)(arg >> 8));   /* Argument[15..8] */
	spi_send((BYTE)arg);				  /* Argument[7..0] */
	n = 0x01;							        /* Dummy CRC + Stop */
	if (cmd == CMD0) n = 0x95;    /* Valid CRC for CMD0(0) */
	if (cmd == CMD8) n = 0x87;    /* Valid CRC for CMD8(0x1AA) */
	spi_send(n);

	/* Receive a command response */
	n = 10;								/* Wait for a valid response in timeout of 10 attempts */
	do {
		res = spi_send(0);
	} while ((res & 0x80) && --n);

	return res;			/* Return with the response value */
}

