/*
 * Copyright (c) 2011-2014 NecoTech.org, All rights reserved.
 */

#include "spi.h"
#include "mp3.h"

DWORD vols[] = { 0x5050, 0x3A3A, 0x1010 };
int currentVol = 1;

DWORD bass[] = { 0x7AF6, 0x44E6, 0x0 };
int curbass = 0;

void set_bass()
{
	mp3_cmd(MC_BASS, bass[curbass++]);
	if (curbass > 2) curbass = 0;
	wait_for_ready();
}

void mp3_init()
{
	UNSELECT_XCS;
	UNSELECT_XDCS;

	PORTD &= ~XRESET;  // reset
	_delay_ms(5);

	PORTD |= XRESET;  // Unlock reset
	_delay_ms(10);

	wait_for_ready();
	mp3_cmd(MC_MODE, _BV(SM_TESTS) | _BV(SM_SDINEW) /*| _BV(SM_SDISHARE)*/);

	wait_for_ready();
	set_clock();

	wait_for_ready();
	set_volume();

	wait_for_ready();
	set_bass();
}

int main__()
{
	DDRD = 0b01110111;
	DDRB = 0b00101110;

	spi_init_low();

	PORTB = _BV(PORTB3);

	PORTD = 0;

	PORTD |= XDCS;
	PORTD |= XCS;

	_delay_ms(10);

	PORTD &= ~XRESET;  // reset
	_delay_ms(5);

	PORTD |= XRESET;   // Unlock reset
	_delay_ms(10);
	wait_for_ready();

	// set to new mode

	mp3_cmd(0x00, _BV(SM_TESTS) | _BV(SM_SDINEW) /*| _BV(SM_SDISHARE)*/);
	wait_for_ready();

	set_clock();
	set_volume();

	return 0;
}

void wait_for_ready()
{
	if (!(PIND & DREQ)) return;

	while (!(PIND & DREQ)) {

#ifdef VS1011E_SLOW_WAIT
		PORTD |= LED;

		if ((PIND & DREQ)) {

			for (int i = 0; i < 3; i++) {
				PORTD |= LED;
				_delay_ms(5);
				PORTD &= ~LED;
				_delay_ms(5);
			}

			PORTD &= ~LED;
			return;
		}

		_delay_ms(1);
		PORTD &= ~LED;
		_delay_ms(1);
#endif

	}
}

#ifdef VS1011E_SPI_SOFT
BYTE spi_byte_soft(BYTE data)
{
	BYTE ret = 0;
	for (int b = 7; b >= 0; b--)
	{
		if (data & (1 << b))
			PORTD |= SI;
		else
			PORTD &= ~SI;

		PORTD |= SCLK;

		ret |= ((PIND & SO) << b);

		PORTD &= ~SCLK;
	}
	return ret;
}
#endif

void send_array(BYTE buf[], unsigned short len)
{
	for (int n = 0; n < len; n++)
#ifdef VS1011E_SPI_SOFT
		spi_byte_soft(buf[n]);
#else
		spi_send(buf[n]);
#endif

}

void mp3_cmd(BYTE reg, DWORD cmd)
{
#ifdef VS1011E_DATA_VALIDATE
	while (1) {
#endif
		BYTE buf[4];
		buf[0] = 2;
		buf[1] = reg;
		buf[2] = cmd >> 8;
		buf[3] = cmd;

		SELECT_XCS;
		send_array(buf, 4);
		UNSELECT_XCS;
		_delay_ms(1);

#ifdef VS1011E_DATA_VALIDATE
		wait_for_ready();
		DWORD retCmd = mp3_read(reg);
		if (retCmd != cmd)
		{
			for (int i = 0; i < 10; i++) {
				PORTB |= _BV(PORTB1);
				_delay_ms(5);
				PORTB &= ~_BV(PORTB1);
				_delay_ms(5);
			}
		}
		else
		{
			return;
		}
	}
#endif
}

DWORD mp3_read(BYTE reg)
{
	BYTE cmd[2];
	cmd[0] = 0x3;
	cmd[1] = reg;

	SELECT_XCS;

	_delay_ms(1);

	send_array(cmd, 2);

#ifdef VS1011E_SPI_SOFT
	DWORD ret = spi_byte_soft(0) << 8;
	ret |= spi_byte_soft(0);
#else
	DWORD ret = spi_recv() << 8;
	ret |= spi_recv();
#endif

	_delay_ms(1);

	UNSELECT_XCS;

	return ret;
}

void set_volume()
{
	mp3_cmd(MC_VOL, vols[currentVol++]);
	if (currentVol > 2) currentVol = 0;
	wait_for_ready();
}

void set_clock()
{
	mp3_cmd(MC_CLOCKF, 0x9800);
	wait_for_ready();
}

void do_sin_test()
{
	BYTE buf[8];
	buf[0] = 0x53;
	buf[1] = 0xEF;
	buf[2] = 0x6E;
	buf[3] = 0x4A;

	for (int i = 0; i < 8 - 4; i++)
		buf[i + 4] = 0;

	while (1)
	{
		UNSELECT_XCS;
		SELECT_XDCS;

		PORTB |= _BV(PORTB1);
		_delay_ms(50);

		send_array(buf, 8);

		PORTB &= ~_BV(PORTB1);

		_delay_ms(50);

		UNSELECT_XDCS;

		_delay_ms(10000);
	}
}
